/** 
 * @file W32_TCP.cpp
 * @brief Win32 TCP/IP communication Program Module 
 * 
 * @author Katayama
 * @date 2013-06-26
 * @version ver 1.00 2013/06/26 katayama
 * @version ver 1.01 2013/08/19 katayama
 * @version ver 1.02 2015/05/14 katayama
 *
 * Copyright (C) 2014 TPIP User Community All rights reserved.
 * ̃t@C̒쌠́ATPIP[U[R~jeB̋Kɏ]
 * gp܂B
 */

// ver 1.01 Domain name corresponding
// ver 1.02 Multiple-port connection correspondence

#include <winsock2.h>
#include <ws2tcpip.h>
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include "W32_TCP.h"
#pragma comment(lib, "ws2_32.lib")


/** \PbgM҂֐
 * @fn     int wait_socket_readable(SOCKET sd, long sec, long usec)
 * @param  sd   : M\Pbg
 * @param  sec  : ^CAEgԁibj
 * @param  usec : @@@@@@@@iʕbj
 * @retval == 1 : M\
 * @retval == 0 : ^CAEg
 * @retval ==-1 : G[
 */
static int wait_socket_readable(SOCKET sd, long sec, long usec)
{
	fd_set rset;
	struct timeval tv;
	int res;

	FD_ZERO(&rset);
	FD_SET(sd, &rset);

	if ( sec | usec ) {
		if ((sec == -1)&&(usec == -1)) {
			sec  = 0;
			usec = 0;
		}
		tv.tv_sec = sec;
		tv.tv_usec = usec;
		res = select(sd + 1, &rset, NULL, NULL, &tv);
	} else {
		res = select(sd + 1, &rset, NULL, NULL, NULL);
	}
	return (res);
}

/** TCP(Server)ʐM|[g֐
 * @fn     SOCKET init_w32tcp_s( int Port_no )
 * @param  Port_no : |[gԍ
 * @retval socket no
 */
SOCKET init_w32tcp_s( int Port_no )
{
	int res;
	int destlen;
	WSADATA data;
	SOCKET destSocket = INVALID_SOCKET;
	SOCKET servSocket = INVALID_SOCKET;
	struct sockaddr_in destSockAddr;
	struct sockaddr_in servSockAddr;

	WSAStartup(MAKEWORD(2,0), &data);

	/* sockaddr_in \̂̃Zbg */
	memset(&servSockAddr, 0, sizeof(servSockAddr));
	servSockAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servSockAddr.sin_port = htons(Port_no);
	servSockAddr.sin_family = AF_INET;

	servSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (servSocket != INVALID_SOCKET) {
		res = bind(servSocket, (struct sockaddr *)&servSockAddr, sizeof(servSockAddr));
		if (res == SOCKET_ERROR) {
			closesocket(servSocket);
			WSACleanup();
			servSocket = INVALID_SOCKET;
			return(servSocket);
		}
		res = listen(servSocket, 1);
		if (res == SOCKET_ERROR) {
			closesocket(servSocket);
			WSACleanup();
			servSocket = INVALID_SOCKET;
			return(servSocket);
		}
		destlen = sizeof(destSockAddr);
		destSocket = accept( servSocket, (struct sockaddr *)&destSockAddr, &destlen);
		closesocket(servSocket);
		servSocket = INVALID_SOCKET;
	}

	return( destSocket ) ;
}

/** TCP(Client)ʐM|[g֐
 * @fn     SOCKET init_w32tcp_c( char* host_adr,int Port_no )
 * @param  host_adr : zXghoAhX,hC
 * @param  Port_no  : |[gԍ
 * @retval socket no
 */
SOCKET init_w32tcp_c(char* host_adr,int Port_no )
{
	int res;
	struct hostent *host;
	WSADATA data;
	SOCKET destSocket = INVALID_SOCKET;
	struct sockaddr_in destSockAddr;

	WSAStartup(MAKEWORD(2,0), &data);
	host = gethostbyname(host_adr);
	/* sockaddr_in \̂̃Zbg */
	memset(&destSockAddr, 0, sizeof(destSockAddr));
	destSockAddr.sin_addr.s_addr = **(unsigned int **)(host->h_addr_list);
	destSockAddr.sin_port = htons(Port_no);
	destSockAddr.sin_family = AF_INET;

	destSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (destSocket != INVALID_SOCKET) {
		res = connect(destSocket, (struct sockaddr *)&destSockAddr, sizeof(destSockAddr));
		if (res == SOCKET_ERROR) {
			closesocket(destSocket);
			WSACleanup();
			destSocket = INVALID_SOCKET;
		}
	}

	return( destSocket ) ;
}


/** TCPʐM|[gI֐
 * @fn     void  close_w32tcp(void)
 * @param  sock : destination socket
 *
 */
void  close_w32tcp(SOCKET sock)
{
	if (sock == INVALID_SOCKET) return;
	closesocket(sock);
	WSACleanup();
	return;
}

/** TCPʐM|[g M֐
 * @fn     int  put_w32tcp(SOCKET sock, char* send_buf, unsigned int send_size)
 * @param  sock      : destination socket
 * @param  send_buf  : Mobt@
 * @param  send_size : Mf[^TCY
 * @retval  == 0     : MG[
 * @retval  >  0     : MTCY
 */
int  put_w32tcp(SOCKET sock, char* send_buf, unsigned int send_size)
{
	DWORD	SendByte ;

	if (sock == INVALID_SOCKET) return(-1);
	SendByte = send(sock, (char*)send_buf, send_size, 0);
	return ( SendByte );
}


/** TCPʐM|[g M֐
 * @fn     int  get_w32tcp(SOCKET sock, char* rcv_buf ,unsigned int r_cnt, int wait_timer)
 * @param  sock       : destination socket
 * @param  rcv_buf    : Mobt@
 * @param  r_cnt      : Mobt@TCY
 * @param  wait_timer : M҂^C}[(us)
 * @retval  == 0      : MȂ
 * @retval  >  0      : MTCY
 * @retval  <  0      : MG[
 */
int  get_w32tcp(SOCKET sock, char* rcv_buf ,unsigned int r_cnt, int wait_timer)
{
	int   res = -1;
	DWORD RecvByte ;
//	int   destSockAddrLen;
	int   rcv_sz;
	int   w_timer;

	RecvByte = 0;
	rcv_sz   = 1;
	w_timer  = wait_timer;
	if (sock == INVALID_SOCKET) return(-1);
	while((rcv_sz > 0)&&(r_cnt > RecvByte)) {
		res = wait_socket_readable(sock, 0, w_timer);
		if (res <= 0) break;
//		destSockAddrLen = sizeof(destSockAddr);
		rcv_sz = recv(sock, (char*)&rcv_buf[RecvByte], r_cnt - RecvByte, 0);
		if (rcv_sz > 0 ) RecvByte = RecvByte + rcv_sz ;
		if (wait_timer == 0) w_timer = 3000*1000;
	}
	if (res < 0) RecvByte = res;
	return ( RecvByte );
}

