/** 
 * @file rLOG_cl.cpp
 * @brief remote Loging client program Module 
 * 
 * @author Katayama
 * @date 2014-10-16
 * @version ver 1.00 2014/10/16 katayama
 * @version ver 1.01 2014/10/17 katayama
 * @version ver 1.02 2015/05/14 katayama
 * @version ver 1.10 2015/07/10 katayama  Log file mode ǉ
 *
 * Copyright (C) 2014 TPIP User Community All rights reserved.
 * ̃t@C̒쌠́ATPIP[U[R~jeB̋Kɏ]
 * gp܂B
 */
#pragma warning(disable:4996)	// ZLeBアÂ֐ɑ΂x̉
#include <time.h>
#include <stdio.h>
#include <string.h>
#include "W32_TCP.h"

static FILE* fd = 0;	// LOG file file descriptor
static char  log_fname[256] = "";
static int   fname_mode = -1;		// LOG file name mode(-1:non, 0:normal, 1:addition of date)
static char  fname_fmt[256] = "";	// file name format(printf)
static char  msg_start[] = "rLOG start";
static char  msg_stop[]  = "rLOG stop";
static char  rcvbf[32];

static HANDLE  hLOG_rcv;
static DWORD   thrID_LOG_rcv;
static volatile int LOG_t_flg;
static int     rLOG_stat = 0;	// rLOG status(0:disconnect, 1:record stop, 2:record start)
static int     rLOG_rq   = 0;	// rLOG request START/STOP(0:STOP, 1:START)
static int     rLOG_on   = 0;	// rLOG on/off(0:off, 1:on)
static int     err_cnt   = 0;	// error(time out) counter
static int     log_end   = 0;   // log ending flag(1: ending)
static SOCKET destSocket = INVALID_SOCKET;


/** check LOG file
 * @fn     int chk_LOG_file(char* fl_name)
 * @param  fl_name : file name
 * @retval != 0 : OK
 * @retval == 0 : error
 */
static int chk_LOG_file(char* fl_name)
{
	int ans = 0;


	strncpy(fname_fmt, "", sizeof(fname_fmt)-1);
	if ((fl_name)) {
		strncpy(fname_fmt, fl_name, sizeof(fname_fmt)-1);
		if (strstr(fl_name, "%02d%02d%02d")) {
			fname_mode = 1;
			ans = 1;
		}
		else {
			fname_mode = 0;
			ans = 1;
		}
	}

	return ans;
}

/** open LOG file
 * @fn     FILE* open_LOG_file()
 * @retval != 0 : file descriptor
 * @retval == 0 : error
 */
static FILE* open_LOG_file(void)
{
	FILE* ans = 0;
	time_t ltime;
	struct tm *newtime;

	strcpy(log_fname, "");

	if (fname_mode == 1) {
		time( &ltime );
		newtime = localtime(&ltime);
		sprintf(log_fname, fname_fmt, newtime->tm_year+1900,
			newtime->tm_mon+1, newtime->tm_mday, newtime->tm_hour, newtime->tm_min, newtime->tm_sec);
	}
	else if (fname_mode == 0) {
		if ((strlen(fname_fmt) > 0)&&(strlen(fname_fmt) < sizeof(log_fname)))
			strcpy(log_fname, fname_fmt);
	}

	if (strlen(log_fname) > 0) {
		if (fname_mode == 1) ans = fopen(log_fname, "wt");
		else ans = fopen(log_fname, "at");
	}
//	printf("\nlog file name [%s] mode:%d <==%d> \n", log_fname, fname_mode, ans);
	return ans;
}

/** rLOG get status
 * @fn  int rLOG_get_stat(void)
 * @retval     : b3-0:rLOG status, b7-4:rLOG request
 *
 */
int rLOG_get_stat(void)
{
	return (rLOG_stat|(rLOG_rq << 4));
}

/** rLOG start
 * @fn  int rLOG_start(void)
 * @retval == 0     : OK
 * @retval != 0     : error
 *
 */
int rLOG_start(void)
{
	int res;
	int ans = 0;

	if (rLOG_stat == 2) return -1;	// rLOG start => exit

	res = put_w32tcp(destSocket, msg_start, strlen(msg_start));
//	printf("send <Log start> = %d[%d] rLOG_on=%d \n", res, rLOG_stat, rLOG_on);
	if (res != strlen(msg_start)) ans = -1;
	else rLOG_rq   = 1;	// rLOG request START
	return ans;
}

/** rLOG stop
 * @fn  int rLOG_stop(void)
 * @retval == 0     : OK
 * @retval != 0     : error
 *
 */
int rLOG_stop(void)
{
	int res;
	int ans = 0;

	if (rLOG_stat != 2) return -1;	// rLOG stop => exit

	res = put_w32tcp(destSocket, msg_stop, strlen(msg_stop));
//	printf("send <Log stop> = %d[%d - %d] \n", res, rLOG_rq, rLOG_stat);
	if (res != strlen(msg_stop)) ans = -1;
	else rLOG_rq   = 0;	// rLOG request STOP
	return ans;
}

/** rLOG recording
 * @fn  int rLOG_record(void)
 * @retval >  0     : OK(receive data size)
 * @retval == 0     : receive time out
 * @retval <  0     : error
 *
 */
int rLOG_record(void)
{
	int res;
	static int pre_rq  = 0;
	int xor_rq;
	char d_buf[9];
	char t_buf[9];

	res = get_w32tcp(destSocket, rcvbf, sizeof(rcvbf)-1, 100*1000) ;
	if (res < 0) {
//		printf("#### Sokect open ERROR ###\n");
		rLOG_stat = 0;		// rLOG status : disconnect
		return -1;	// exit
	}

	xor_rq = pre_rq ^ rLOG_rq;

	if (xor_rq & rLOG_rq) {	// rLOG_rq (STOP -> START)
		if (!fd) fd = open_LOG_file();

		if ((fname_mode == 0)&&(fd)) {
			_strdate( d_buf );
			_strtime( t_buf );
			fprintf(fd, "--- start of log data ---  *** %s %s ***\n",d_buf ,t_buf);
		}
		rLOG_on = 1;
//		printf("--- start of log data ---  \n");
	}

	if (res > 0) {
		err_cnt    = 0;		// reset error count
		rLOG_stat  = 2;		// rLOG status : start
		rcvbf[res] = 0;

		if (fd) {
			fputs(rcvbf, fd);
		}
		else {
			printf("%s", rcvbf);
//			if (rLOG_on == 0) printf("\n#### error ### xor_rq=%d pre_rq=%d rLOG_rq=%d \n", xor_rq, pre_rq, rLOG_rq);
		}
	} else {
		if (log_end) {
			if (fd) {
				if (fname_mode == 0) fprintf(fd, "--- end of log data ---  \n\n");
				fclose(fd);
				fd = NULL;
			}
//			printf("--- end of log data ---  \n");
			rLOG_on = 0;
			log_end = 0;
		}
		if (err_cnt++ > 10) {
			err_cnt   = 10;
			if (rLOG_stat != 1) rLOG_stat = 1;
		}
	}
	if (xor_rq & pre_rq) {	// rLOG_rq (START -> STOP)
		log_end = 1;		// request log-end print
	}
	pre_rq = rLOG_rq;
	return res;
}

/** rLOG receive Task
 * @fn   recv_task(void)
 *
 */
void recv_task(void)
{
	int res;

	LOG_t_flg = 1;
	res = put_w32tcp(destSocket, msg_stop, strlen(msg_stop));
//	printf("recv_task --- start res =%d ---\n", res);
	while (LOG_t_flg > 0){
		rLOG_record();
	}
	rLOG_stat = 0;
	LOG_t_flg = 0;
}


/** rLOG initialize
 * @fn  int rLOG_init(char* host_ip, int port_no, char* fl_name)
 * @param  host_ip  : Host IP address
 * @param  port_no  : rLOG TCP port no
 * @param  fl_name  : log file name
 * @retval == 0     : OK
 * @retval != 0     : error
 *
 */
int rLOG_init(char* host_ip, int port_no, char* fl_name)
{
	int ans = 0;
	int i;

	destSocket = init_w32tcp_c( host_ip, port_no );
	if (destSocket == INVALID_SOCKET) ans |= 0x01;

	if ((fl_name)) {
		chk_LOG_file(fl_name);
	}

	if (ans == 0) {
		LOG_t_flg = 0;
		hLOG_rcv = CreateThread(0,0,(LPTHREAD_START_ROUTINE)recv_task, NULL, 0, &thrID_LOG_rcv);
		for ( i = 0; i < 10; i++ ) {
			if (LOG_t_flg == 1) break;
			Sleep(10);
		}
		if (LOG_t_flg <= 0) ans |= 0x04;
	}

	return (ans);
}

/** rLOG ending
 * @fn  int rLOG_end(void)
 * @retval == 0     : OK
 * @retval != 0     : error
 *
 */
int rLOG_end(void)
{
	int ans = 0;
	int i;

	if (LOG_t_flg > 0) {
		LOG_t_flg = -1;
		for ( i = 0; i < 100; i++ ) {
			if (LOG_t_flg == 0) break;
			Sleep(100);
		}
		if (LOG_t_flg != 0) ans |= 0x01;
	}

	close_w32tcp(destSocket);
	destSocket = INVALID_SOCKET;
	if (fd) {
		if (fname_mode == 0) fprintf(fd, "--- end of log data ---  \n\n");
		ans = fclose(fd);
		fd = NULL;
	}
	return ans;
}

